<?php

namespace Drupal\tag1quo\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Link;
use Drupal\Component\Serialization\Json;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Admin controller for the tag1quo module.
 */
class AdminController extends ControllerBase {

  /** @var \Psr\Log\LoggerInterface $logger  */
  protected $logger;

  /**
   * The date formatter service.
   *
   * @var \Drupal\Core\Datetime\DateFormatterInterface
   */
  protected $dateFormatter;

  public static function create(ContainerInterface $container) {
    return new static($container->get('logger.factory')->get('tag1quo'), $container->get('date.formatter'));
  }

  public function __construct(LoggerInterface $logger, DateFormatterInterface $date_formatter) {
    $this->logger = $logger;
    $this->dateFormatter = $date_formatter;
  }


  /**
   * Admin review tab page
   */
  public function adminReview() {
    $status = tag1quo_get_status();

    // If not enabled, just return $build saying so.
    if (!empty($build = $this->reportStatusFailure($status))) {
      return $build;
    };

    $header = [
      $this->t('Name'),
      $this->t('Type'),
      $this->t('Version'),
      $this->t('Schema version'),
      $this->t('Enabled'),
      $this->t('Filename'),
    ];

    $rows = [];
    $enabled = $status['field_json_data'][0]['value'];
    foreach ($enabled as $project) {
      if (isset($project['info'])) {
        $info = unserialize($project['info']);
      }
      else {
        $info = [];
      }
      $row = [
        $project['name'],
        $project['type'],
        isset($info['version']) ? $info['version'] : $this->t('undefined'),
        $project['schema_version'],
        $project['status'],
        $project['filename'],
      ];
      $rows[] = $row;
    }

    $build['status_table'] = [
      '#theme' => 'table',
      '#header' => $header,
      '#rows' => $rows,
    ];

    $status['field_json_data'] = [
      [
        'value' => Json::encode($status['field_json_data'][0]['value'])
      ]
    ];

    // Show the actual JSON that's being sent to Tag1 Quo.
    $build['field_json_data'] = [
      '#type' => 'details',
      '#title' => $this->t('Raw data'),
      '#open' => FALSE,
      '#markup' => '<div class="container-inline">' . Json::encode($status) . '</div>',
      '#attributes' => [],
      '#children' => '',
    ];

    return $build;
  }

  /**
   * admin status page.
   */
  public function adminStatus() {

    $status = tag1quo_get_status();

    // If not enabled, just return $build saying so.
    if (!empty($build = $this->reportStatusFailure($status))) {
      return $build;
    }

    $next_update = tag1quo_next_update();
    if ($next_update <= 0) {
      $build[] = [
        '#type' => 'item',
        '#markup' => $this->t('A heartbeat will be sent to Tag1 Quo the next time cron runs.')
      ];
    }
    else {
      $build[] = [
        '#type' => 'item',
        '#markup' => $this->t('A heartbeat will be sent to Tag1 Quo in @time.', [
          '@time' => $this->dateFormatter->formatInterval($next_update)
        ])
      ];
    }
    $build[] = [
      '#type' => 'item',
      '#markup' => Link::createFromRoute($this->t('Send manually'), 'tag1quo.admin_send_manually')
        ->toString()
    ];


    $last_update = \Drupal::state()->get('tag1quo_last_update', 0);
    if (!$last_update) {
      $build[] = [
        '#type' => 'item',
        '#markup' => $this->t('No heartbeat has ever been sent. Please be sure you have cron configured correctly.')
      ];
    }
    else {
      $time = REQUEST_TIME;
      $build[] = [
        '#type' => 'item',
        '#markup' => $this->t('The last heartbeat was sent @time ago.', [
          '@time' => $this->dateFormatter->formatInterval($time - $last_update)
        ])
      ];
      if (($time - $last_update) > TAG1QUO_HEARTBEAT_FREQUENCY * 2) {
        drupal_set_message($this->t('No recent heartbeat has been sent. Please be sure you have cron configured correctly.'), 'error');
      }
    }

    return $build;
  }

  /**
   * Provide $build for case when not enabled.
   */
  public function reportStatusFailure($status) {
    $build = [];
    if (isset($status['error'])) {
      $config_page = Link::createFromRoute($this->t('configuration page'), 'tag1quo.admin_settings')->toString();
      $build[] = [
        '#type' => 'item',
        '#markup' => $this->t('No information is currently being shared with Tag1 Quo.')
      ];
      if ($status['error'] == TAG1QUO_ERROR_DISABLED) {
        $build[] = [
          '#type' => 'item',
          '#markup' => $this->t('Reporting to Tag1 Quo is disabled on the @config_page.', ['@config_page' => $config_page]),
        ];
      }
      if ($status['error'] == TAG1QUO_ERROR_NO_TOKEN) {
        $build[] = [
          '#type' => 'item',
          '#markup' => $this->t('The Tag1 Quo token has not yet been configured on the @config_page.', ['@config_page' => $config_page]),
        ];
      }
    }
    return $build;
  }

  /**
   * Send manually function
   */
  public function sendManually() {
    $result = tag1quo_send_report();

    switch ($result) {
      case TAG1QUO_SUCCESS:
        drupal_set_message('Module status reported to Tag1 Quo.');
        break;
      case TAG1QUO_ERROR_DISABLED:
        drupal_set_message($this->t('Reporting is disabled. You can enable it in the Configuration tab.'), 'error');
        break;
      case TAG1QUO_ERROR_NO_TOKEN:
        drupal_set_message($this->t('Manual update aborted, you must first configure a token in the Configuration tab.'), 'error');
        break;
      case TAG1QUO_ERROR_ON_PHONE_HOME:
        drupal_set_message($this->t('Failed to report module status to Tag1 Quo. Check the Drupal logs for more information.'), 'error');
        $this->logger->info($this->t('Manual module status report attempt failed.'));
        break;
      default:
        drupal_set_message($this->t('Unknown error: See Drupal logs for more information.'));
    }
    return $this->redirect('tag1quo.admin_status');
  }

}
